# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: depot/core/v1/build.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x64\x65pot/core/v1/build.proto\x12\rdepot.core.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"%\n\x11ShareBuildRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\"\'\n\x12ShareBuildResponse\x12\x11\n\tshare_url\x18\x01 \x01(\t\"+\n\x17StopSharingBuildRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\"\x1a\n\x18StopSharingBuildResponse\"u\n\x11ListBuildsRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x16\n\tpage_size\x18\x02 \x01(\x05H\x00\x88\x01\x01\x12\x17\n\npage_token\x18\x03 \x01(\tH\x01\x88\x01\x01\x42\x0c\n\n_page_sizeB\r\n\x0b_page_token\"S\n\x12ListBuildsResponse\x12$\n\x06\x62uilds\x18\x01 \x03(\x0b\x32\x14.depot.core.v1.Build\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"#\n\x0fGetBuildRequest\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\"7\n\x10GetBuildResponse\x12#\n\x05\x62uild\x18\x01 \x01(\x0b\x32\x14.depot.core.v1.Build\"\xdb\x04\n\x05\x42uild\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12+\n\x06status\x18\x02 \x01(\x0e\x32\x1b.depot.core.v1.Build.Status\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x33\n\nstarted_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x88\x01\x01\x12\x34\n\x0b\x66inished_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x01\x88\x01\x01\x12#\n\x16\x62uild_duration_seconds\x18\x06 \x01(\x05H\x02\x88\x01\x01\x12#\n\x16saved_duration_seconds\x18\x07 \x01(\x05H\x03\x88\x01\x01\x12\x19\n\x0c\x63\x61\x63hed_steps\x18\x08 \x01(\x05H\x04\x88\x01\x01\x12\x18\n\x0btotal_steps\x18\t \x01(\x05H\x05\x88\x01\x01\"\x82\x01\n\x06Status\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x12\n\x0eSTATUS_RUNNING\x10\x01\x12\x11\n\rSTATUS_FAILED\x10\x02\x12\x12\n\x0eSTATUS_SUCCESS\x10\x03\x12\x10\n\x0cSTATUS_ERROR\x10\x04\x12\x13\n\x0fSTATUS_CANCELED\x10\x05\x42\r\n\x0b_started_atB\x0e\n\x0c_finished_atB\x19\n\x17_build_duration_secondsB\x19\n\x17_saved_duration_secondsB\x0f\n\r_cached_stepsB\x0e\n\x0c_total_steps2\xe6\x02\n\x0c\x42uildService\x12Q\n\nShareBuild\x12 .depot.core.v1.ShareBuildRequest\x1a!.depot.core.v1.ShareBuildResponse\x12\x63\n\x10StopSharingBuild\x12&.depot.core.v1.StopSharingBuildRequest\x1a\'.depot.core.v1.StopSharingBuildResponse\x12Q\n\nListBuilds\x12 .depot.core.v1.ListBuildsRequest\x1a!.depot.core.v1.ListBuildsResponse\x12K\n\x08GetBuild\x12\x1e.depot.core.v1.GetBuildRequest\x1a\x1f.depot.core.v1.GetBuildResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'depot.core.v1.build_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SHAREBUILDREQUEST']._serialized_start=77
  _globals['_SHAREBUILDREQUEST']._serialized_end=114
  _globals['_SHAREBUILDRESPONSE']._serialized_start=116
  _globals['_SHAREBUILDRESPONSE']._serialized_end=155
  _globals['_STOPSHARINGBUILDREQUEST']._serialized_start=157
  _globals['_STOPSHARINGBUILDREQUEST']._serialized_end=200
  _globals['_STOPSHARINGBUILDRESPONSE']._serialized_start=202
  _globals['_STOPSHARINGBUILDRESPONSE']._serialized_end=228
  _globals['_LISTBUILDSREQUEST']._serialized_start=230
  _globals['_LISTBUILDSREQUEST']._serialized_end=347
  _globals['_LISTBUILDSRESPONSE']._serialized_start=349
  _globals['_LISTBUILDSRESPONSE']._serialized_end=432
  _globals['_GETBUILDREQUEST']._serialized_start=434
  _globals['_GETBUILDREQUEST']._serialized_end=469
  _globals['_GETBUILDRESPONSE']._serialized_start=471
  _globals['_GETBUILDRESPONSE']._serialized_end=526
  _globals['_BUILD']._serialized_start=529
  _globals['_BUILD']._serialized_end=1132
  _globals['_BUILD_STATUS']._serialized_start=884
  _globals['_BUILD_STATUS']._serialized_end=1014
  _globals['_BUILDSERVICE']._serialized_start=1135
  _globals['_BUILDSERVICE']._serialized_end=1493
# @@protoc_insertion_point(module_scope)
